package my.petstore.service;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import my.petstore.entity.Inventory;
import my.petstore.entity.LineItem;
import my.petstore.entity.OrderData;

import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Repository
@Transactional
public class OrderServiceImpl implements OrderService {

    @PersistenceContext
    private EntityManager em;

    public void storeOrderData(OrderData orderData) throws OutOfOrderException {
        for (LineItem lineItem : orderData.getLineItems()) {
            int decrement = lineItem.getQuantity();
            Inventory inventory = lineItem.getItem().getInventory();
            int current = inventory.decrementQuantity(decrement);
            if (current < 0) {
                throw new OutOfOrderException();
            }
            this.em.merge(inventory);
        }

        this.em.merge(orderData);
    }


    @Transactional(readOnly = true)
    public OrderData getOrderData(Integer orderId) {
        OrderData orderData = this.em.find(OrderData.class, orderId);
        return orderData != null ? orderData : OrderData.NULL;
    }

    @Transactional(readOnly = true)
    public List<OrderData> getOrderData() {
        return this.em.createQuery("SELECT orderData FROM OrderData orderData").getResultList();
    }
}
