package my.petstore.validation;

import my.petstore.entity.OrderData;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

@Component("OrderValidator")
public class OrderValidator implements Validator {
    
    @Override
    public boolean supports(Class clazz) {
        return OrderData.class.isAssignableFrom(clazz);
    }

    @Override
    public void validate(Object obj, Errors errors) {
        validateItems((OrderData) obj, errors);
        validateCreditCard((OrderData) obj, errors);
        validateShippingAddress((OrderData) obj, errors);
    }

    private void validateItems(OrderData orderData, Errors errors) {
        if (orderData.getLineItems().isEmpty()) {
            errors.rejectValue("lineItems", "should have items");
        }
    }
    
    private void validateCreditCard(OrderData orderData, Errors errors) {
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "creditCard", "CCN_REQUIRED", "FAKE (!) credit card number required.");
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "expiryDate", "EXPIRY_DATE_REQUIRED", "Expiry date is required.");
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "cardType", "CARD_TYPE_REQUIRED", "Card type is required.");
    }

    private void validateShippingAddress(OrderData orderData, Errors errors) {
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "shipToName", "NAME_REQUIRED", "Shipping Info: name is required.");
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "shipAddress", "ADDRESS_REQUIRED", "Shipping Info: address is required.");
    }
}
