package my.petstore.web;

import java.util.ArrayList;

import javax.servlet.http.HttpSession;

import my.petstore.entity.Category;
import my.petstore.entity.Product;
import my.petstore.service.ProductService;
import my.petstore.util.PagedListUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.support.PagedListHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Controller
public class CategoryController {

    @Autowired
    private ProductService productService;

    @RequestMapping(value="/categories", method=RequestMethod.GET)
    public String categories(@RequestParam(defaultValue="") String page, HttpSession session, Model model) {
        Category category = (Category) session.getAttribute("ViewProductAction_category");
        PagedListHolder productList = (PagedListHolder) session.getAttribute("ViewProductAction_productList");
        if (category == null || productList == null) {
            throw new IllegalStateException("Cannot find pre-loaded category and product list");
        }
        PagedListUtil.movePage(productList, page);

        model.addAttribute("category", category);
        model.addAttribute("productList", productList);
        return "Category";
    }

    @RequestMapping(value="/categories/{categoryId}", method=RequestMethod.GET)
    public String category(@PathVariable Integer categoryId, @RequestParam(required=false) String page, HttpSession session, Model model) {
        Category category = productService.getCategory(categoryId);
        PagedListHolder productList = new PagedListHolder(new ArrayList<Product>(category.getProducts()));
        productList.setPageSize(4);
        session.setAttribute("ViewProductAction_category", category);
        session.setAttribute("ViewProductAction_productList", productList);

        model.addAttribute("category", category);
        model.addAttribute("productList", productList);
        return "Category";
    }
}
