package my.petstore.service;

import static org.junit.Assert.assertThat;
import static org.hamcrest.Matchers.*;

import my.petstore.entity.Cart;
import my.petstore.entity.Item;
import my.petstore.entity.OrderData;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.transaction.annotation.Transactional;

@ContextConfiguration(locations = {"classpath:spring/applicationContext-jpa.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class OrderServiceTest {

    @Autowired
    protected OrderService orderService;

    @Autowired
    protected ProductService productService;

    @Test
    public void storeOrderDataAndCheckInventory() {
        storeOrderData();
        checkInventory();
    }

    @Transactional
    private void storeOrderData() {
        Cart cart = new Cart();
        cart.addItem(productService.getItem(1), true);
        cart.addItem(productService.getItem(2), true);

        OrderData orderData = new OrderData(cart);
        orderData.setShipToName("foo");
        orderData.setShipAddress("bar");

        orderService.storeOrderData(orderData);
    }

    @Transactional(readOnly = true)
    private void checkInventory() {
        Item item = productService.getItem(1);
        assertThat(item.getInventory().getQuantity(), is(9));
    }


    @Test
    public void emptyOrder() {
        storeEmpty();
    }

    @Transactional
    private void storeEmpty() {
        Cart cart = new Cart();

        OrderData orderData = new OrderData(cart);
        orderData.setShipToName("foo");
        orderData.setShipAddress("bar");

        orderService.storeOrderData(orderData);
    }


    @Test(expected = OrderService.OutOfOrderException.class)
    public void outOfOrder() {
        final int itemId = 3;
        Item item = productService.getItem(itemId);
        assertThat(productService.isItemInStock(itemId), is(true));

        storeOrderDataWithNum(itemId, 1);
        assertThat(productService.isItemInStock(itemId), is(true));

        storeOrderDataWithNum(itemId,9);
        assertThat(productService.isItemInStock(itemId), is(false));

        storeOrderDataWithNum(itemId, 1);
    }

    @Transactional
    private void storeOrderDataWithNum(int itemId, int itemNum) {
        Cart cart = new Cart();
        for (int i = 0; i < itemNum; i++) {
            cart.addItem(productService.getItem(itemId), true);
        }

        OrderData orderData = new OrderData(cart);
        orderData.setShipToName("foo");
        orderData.setShipAddress("bar");

        orderService.storeOrderData(orderData);
    }
}
